#include <blitz/array.h>

/* Contour lines for function evaluated on a grid.

  Copyright (C) 2004 Victor Munoz

  Based on contour plot routine (plcont.c) in PLPlot package
  http://plplot.org/

  Copyright (C) 1995, 2000, 2001 Maurice LeBrun
  Copyright (C) 2000, 2002 Joao Cardoso
  Copyright (C) 2000, 2001, 2002, 2004  Alan W. Irwin
  Copyright (C) 2004  Andrew Ross

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Library Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

/*

ChangeLog

Version 0.1, 13 May 2004
  First release. Contour algorithm from PLPlot. C++ interface, use of
  Blitz++

*/

/*
   Contours are calculated by the function:
                cl_cont(X,Y,Z,clevel,nlevels);
   
   The grid has XPTS points in x-axis, given in array X.
                YPTS points in y-axis, given in array Y.
   The function f(X,Y) is given in the array Z (YPTS rows, XPTS columns),
   so that
                Z(i,j) = f(X(j),Y(i)).
		
   Contour levels are given in array clevel. nlevels is the number
   of levels.

   Result is returned in the 2-column array contourc. If contour
   levels are [c1,c2,...], consisting of [n1,n2,...] points 
   respectively, contourc has the form:

   c1    n1
   x1,1  y1,1
   x2,1  y2,1
   ...
   xn1,1 yn1,1
   c2    n2
   x1,2  y2,1
   x2,2  y2,2
   ... 
   
*/		


/* Due to blitz features, codes like:
   function(Array<float> c){
      c.f(); // f=resizeAndPreserve, f= free, etc
   }
   do not have effect. If c is a global variable, and is not passed
   as an argument to function(), it works. 

   Functions can modify arrays contents inside them, but cannot change their
   sizes, it seems.
*/
   

using namespace blitz;

static Array<float,2> this_contour;
static Array<float,2> contourc;

Array<float,2> cl_cont(Array<float,1> /*X*/,Array<float,1> /*Y*/,
		       Array<float,2> /*Z*/,Array<float,1> /*clevel*/,
		       int /*nlevels*/);
void cl_cntr(Array<float,1>,Array<float,1>,Array<float,2>,float);
void cl_drawcn(Array<float,1> /*X*/,Array<float,1> /*Y*/,
	       Array<float,2> /*Z*/,float /*flev*/,int /*krow*/,int /*kcol*/,
	       float /*lastx*/,float /*lasty*/,int /*startedge*/,
	       Array<int,2> /*ipts*/);
void cl_add_point(float /*x*/,float /*y*/);
void cl_start_contour(float /*flev*/,float /*x*/,float /*y*/);
void cl_end_contour();
